/*
 *  main.c
 *  SkeletonAddition
 */

#include <Carbon/Carbon.h>
#include "TermCodes.h"

#ifdef __cplusplus
	#define EXTERN_C extern "C"
#else
	#define EXTERN_C
#endif


UInt32			gAdditionReferenceCount = 0;
CFBundleRef		gAdditionBundle;

// =============================================================================
// == Entry points.

static OSErr InstallMyEventHandlers();
static void RemoveMyEventHandlers();

EXTERN_C OSErr SAInitialize(CFBundleRef theBundle)
{
	/*  Typically, scripting additions either totally succeed to load or totally fail.  This is usually, but not always, the right thing to do -- if you had an addition where part of it relied on a shared library, but part didn't, you might want to run in a reduced mode if the library could not be found. */

	gAdditionBundle = theBundle;  // no retain needed.

	// Any other setup you need here...

	return InstallMyEventHandlers();
}

EXTERN_C void SATerminate()
{
	// Release anything you allocated in SAInitialize here...

	RemoveMyEventHandlers();
}

EXTERN_C Boolean SAIsBusy()
{
	return gAdditionReferenceCount != 0;
}

// =============================================================================

OSErr MyEventHandler(const AppleEvent *ev, AppleEvent *reply, long refcon)
{
	++gAdditionReferenceCount;  // increment the reference count first thing!
	
	// Go do that voodoo that you do so well...
	
	--gAdditionReferenceCount;  // don't forget to decrement the reference count when you leave!

	return noErr;
}

// -----------------------------------------------------------------------------
// -- Event handler data.

struct AEEventHandlerInfo {
	FourCharCode			evClass, evID;
	AEEventHandlerProcPtr	proc;
};
typedef struct AEEventHandlerInfo AEEventHandlerInfo;

static const AEEventHandlerInfo gEventInfo[] = {
	{ kSkeletonSuite, kSkeletonEvent, MyEventHandler }
	// Add more suite/event/handler triplets here if you define more than one command.
};
#define kEventHandlerCount  (sizeof(gEventInfo) / sizeof(AEEventHandlerInfo))

static AEEventHandlerUPP gEventUPPs[kEventHandlerCount];

// =============================================================================

static OSErr InstallMyEventHandlers()
{
	OSErr		err;
	size_t		i;
	
	for (i = 0; i < kEventHandlerCount; ++i) {
		if ((gEventUPPs[i] = NewAEEventHandlerUPP(gEventInfo[i].proc)) != NULL)
			err = AEInstallEventHandler(gEventInfo[i].evClass, gEventInfo[i].evID, gEventUPPs[i], 0, true);
		else
			err = memFullErr;
		
		if (err != noErr) {
			SATerminate();  // Call the termination function ourselves, because the loader won't once we fail.
			return err;
		}
	}

	return noErr; 
}

// -----------------------------------------------------------------------------

static void RemoveMyEventHandlers()
{
	OSErr		err;
	size_t		i;

	for (i = 0; i < kEventHandlerCount; ++i) {
		if (gEventUPPs[i] &&
			(err = AERemoveEventHandler(gEventInfo[i].evClass, gEventInfo[i].evID, gEventUPPs[i], true)) == noErr)
		{
			DisposeAEEventHandlerUPP(gEventUPPs[i]);
			gEventUPPs[i] = NULL;
		}
	}
}
