// MyOraclePlugIn.java
// Copyright (c) 1999, Apple Computer, Inc. All rights reserved.

/*
THIS SOFTWARE IS FURNISHED ON AN "AS-IS" BASIS. APPLE MAKES NO WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, AS TO ANY MATTER WHATSOEVER, INCLUDING WITHOUT LIMITATION
THE CONDITION, MERCHANTABILITY, OR FITNESS FOR ANY PARTICULAR PURPOSE OF THIS SOFTWARE.
APPLE DOES NOT ASSUME ANY LIABILITY REGARDING USE OF, OR ANY DEFECT IN, THIS SOFTWARE.
IN NO EVENT SHALL APPLE BE LIABLE FOR ANY INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, EVEN IF IT HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import com.apple.cocoa.foundation.*;
import com.apple.yellow.eoaccess.*;
import com.apple.yellow.eocontrol.*;
import java.util.*;
import java.sql.*;

public class MyOraclePlugIn extends OraclePlugIn {
    
    public MyOraclePlugIn(JDBCAdaptor adaptor) {
        super(adaptor);
        // Just for debugging to show that the new plugin is being used by EOModeler
        System.out.println("Constructing MyOraclePlugIn for adaptor " + adaptor);
    }

    public Class defaultExpressionClass() {
        return MyOracleExpression.class;
    }

    public EOSynchronizationFactory createSynchronizationFactory() {
        return new MyOraclePlugIn.MyOracleSynchronizationFactory(_adaptor);
    }

    public String defaultDriverName() {
        return "oracle.jdbc.driver.OracleDriver";
    }
    

    public static class MyOracleExpression extends OraclePlugIn.OracleExpression {

        // BEGIN_BRIDGE
        private static MyOracleExpression _sharedInstance = null;
        private static MyOracleSynchronizationFactory _sharedSyncFactory = null;
    
        public static EOSQLExpression sharedInstance() {
            if (_sharedInstance == null) {
                _sharedInstance = new MyOracleExpression();
            }
            return _sharedInstance;
        }
    
        public static EOSynchronizationFactory sharedSyncFactory() {
            if (_sharedSyncFactory == null) {
                _sharedSyncFactory = new MyOracleSynchronizationFactory();
            }
            return _sharedSyncFactory;
        }
        // END_BRIDGE

        public MyOracleExpression(EOEntity entity) {
            super(entity);
        }

        // BEGIN_BRIDGE
        public MyOracleExpression() {
            super();
        }
        // END_BRIDGE

        // BEGIN_BRIDGE
        public Class _synchronizationFactoryClass() {
            return MyOracleSynchronizationFactory.class;
        }
        // END_BRIDGE

    }


    public static class MyOracleSynchronizationFactory extends OraclePlugIn.OracleSynchronizationFactory {
    
        public MyOracleSynchronizationFactory(EOAdaptor adaptor) {
            super(adaptor);
        }

        // BEGIN_BRIDGE
        public MyOracleSynchronizationFactory() {
            super();
        }
        // END_BRIDGE
        
        public boolean  supportsSchemaSynchronization() {
            return true;
        }

    }
}
